/*
 * Copyright 2005 - 2014  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * MAPIErrors.cpp
 * Definition of GetMAPIErrorMessage()
 */

#include <cstring>
#include "MAPIErrors.h"
#include "mapidefs.h"

#include "mapicode.h"
#include "mapiext.h"

typedef struct tagMAPIErrorTranslateRecord
{
	HRESULT errorCode;
    const char* errorMessage;
} MAPIErrorTranslateRecord;

static MAPIErrorTranslateRecord MAPIErrorCodes[] =
{
    { MAPI_E_CALL_FAILED,                   "Call failed" },
    { MAPI_E_NOT_ENOUGH_MEMORY,             "Not enough memory" },
    { MAPI_E_INVALID_PARAMETER,             "Invalid parameter" },
    { MAPI_E_INTERFACE_NOT_SUPPORTED,       "Interface not supported" },
    { MAPI_E_NO_ACCESS,                     "No access" },
    { MAPI_E_NO_SUPPORT,                    "No support" },
    { MAPI_E_BAD_CHARWIDTH,                 "Bad character width" },
    { MAPI_E_STRING_TOO_LONG,               "String too long" },
    { MAPI_E_UNKNOWN_FLAGS,                 "Unknown flags" },
    { MAPI_E_INVALID_ENTRYID,               "Invalid entry" },
    { MAPI_E_INVALID_OBJECT,                "Invalid object" },
    { MAPI_E_OBJECT_CHANGED,                "Object changed" },
    { MAPI_E_OBJECT_DELETED,                "Object deleted" },
    { MAPI_E_BUSY,                          "Busy" },
    { MAPI_E_NOT_ENOUGH_DISK,               "Not enough disk" },
    { MAPI_E_NOT_ENOUGH_RESOURCES,          "Not enough resources" },
    { MAPI_E_NOT_FOUND,                     "Not found" },
    { MAPI_E_VERSION,                       "Version" },
    { MAPI_E_LOGON_FAILED,                  "Logon failed" },
    { MAPI_E_SESSION_LIMIT,                 "Session limit" },
    { MAPI_E_USER_CANCEL,                   "Use cancel" },
    { MAPI_E_UNABLE_TO_ABORT,               "Unable to abort" },
    { MAPI_E_NETWORK_ERROR,                 "Network error" },
    { MAPI_E_DISK_ERROR,                    "Disk error" },
    { MAPI_E_TOO_COMPLEX,                   "Too complex" },
    { MAPI_E_BAD_COLUMN,                    "Bad column" },
    { MAPI_E_EXTENDED_ERROR,                "Extended error" },
    { MAPI_E_COMPUTED,                      "Computed" },
    { MAPI_E_CORRUPT_DATA,                  "Corrupt data" },
    { MAPI_E_UNCONFIGURED,                  "Unconfigured" },
    { MAPI_E_FAILONEPROVIDER,               "Failoneprovider" },
    { MAPI_E_UNKNOWN_CPID,                  "Unknown CPID" },
    { MAPI_E_UNKNOWN_LCID,                  "Unknown LCID" },
    { MAPI_E_PASSWORD_CHANGE_REQUIRED,      "Password change required" },
    { MAPI_E_PASSWORD_EXPIRED,              "Password expired" },
    { MAPI_E_INVALID_WORKSTATION_ACCOUNT,   "Invalid workstation account" },
    { MAPI_E_INVALID_ACCESS_TIME,           "Invalid access time" },
    { MAPI_E_ACCOUNT_DISABLED,              "Account disabled" },
    { MAPI_E_END_OF_SESSION,                "End of session" },
    { MAPI_E_UNKNOWN_ENTRYID,               "Unknown entryid" },
    { MAPI_E_MISSING_REQUIRED_COLUMN,       "Missing required column" },
    { MAPI_W_NO_SERVICE,                    "No service" },
    { MAPI_E_BAD_VALUE,                     "Bad value" },
    { MAPI_E_INVALID_TYPE,                  "Invalid type" },
    { MAPI_E_TYPE_NO_SUPPORT,               "Type no support" },
    { MAPI_E_UNEXPECTED_TYPE,               "Unexpected_type" },
    { MAPI_E_TOO_BIG,                       "Too big" },
    { MAPI_E_DECLINE_COPY,                  "Decline copy" },
    { MAPI_E_UNEXPECTED_ID,                 "Unexpected id" },
    { MAPI_W_ERRORS_RETURNED,               "Errors returned" },
    { MAPI_E_UNABLE_TO_COMPLETE,            "Unable to complete" },
    { MAPI_E_TIMEOUT,                       "Timeout" },
    { MAPI_E_TABLE_EMPTY,                   "Table empty" },
    { MAPI_E_TABLE_TOO_BIG,                 "Table too big" },
    { MAPI_E_INVALID_BOOKMARK,              "Invalid bookmark" },
    { MAPI_W_POSITION_CHANGED,              "Position changed" },
    { MAPI_W_APPROX_COUNT,                  "Approx count" },
    { MAPI_E_WAIT,                          "Wait" },
    { MAPI_E_CANCEL,                        "Cancel" },
    { MAPI_E_NOT_ME,                        "Not me" },
    { MAPI_W_CANCEL_MESSAGE,                "Cancel message" },
    { MAPI_E_CORRUPT_STORE,                 "Corrupt store" },
    { MAPI_E_NOT_IN_QUEUE,                  "Not in queue" },
    { MAPI_E_NO_SUPPRESS,                   "No suppress" },
    { MAPI_E_COLLISION,                     "Collision" },
    { MAPI_E_NOT_INITIALIZED,               "Not initialized" },
    { MAPI_E_NON_STANDARD,                  "Non standard" },
    { MAPI_E_NO_RECIPIENTS,                 "No recipients" },
    { MAPI_E_SUBMITTED,                     "Submitted" },
    { MAPI_E_HAS_FOLDERS,                   "Has folders" },
    { MAPI_E_HAS_MESSAGES,                  "Has messages" },
    { MAPI_E_FOLDER_CYCLE,                  "Folder cycle" },
    { MAPI_W_PARTIAL_COMPLETION,            "Partial completion" },
    { MAPI_E_AMBIGUOUS_RECIP,               "Ambiguous recipient" },
    { MAPI_E_STORE_FULL,                    "Store full" },
};

/**
 * Get a string describing a MAPI error
 *
 * @param[in] errorCode MAPI error code
 * @retval    C string describing error code, or NULL if not found
 */
const char* GetMAPIErrorMessage(unsigned int errorCode)
{
    const char* retval = "Unknown error code";
    size_t tableSize = sizeof(MAPIErrorCodes) / sizeof(MAPIErrorCodes[0]);
    for (size_t i = 0; i < tableSize; i++)
    {
        if (MAPIErrorCodes[i].errorCode == errorCode)
        {
            retval = MAPIErrorCodes[i].errorMessage;
            break;
        }
    }
    return retval;
}
